@@ -533,16 +533,16 @@ class OPF(object):  # {{{
     authors_path    = XPath('descendant::*[re:match(name(), "creator", "i") and (@role="aut" or @opf:role="aut" or (not(@role) and not(@opf:role)))]')
     bkp_path        = XPath('descendant::*[re:match(name(), "contributor", "i") and (@role="bkp" or @opf:role="bkp")]')
     tags_path       = XPath('descendant::*[re:match(name(), "subject", "i")]')
-    isbn_path       = XPath('descendant::*[re:match(name(), "identifier", "i") and '+
+    isbn_path       = XPath('descendant::*[re:match(name(), "identifier", "i") and '
                             '(re:match(@scheme, "isbn", "i") or re:match(@opf:scheme, "isbn", "i"))]')
     pubdate_path    = XPath('descendant::*[re:match(name(), "date", "i")]')
-    raster_cover_path = XPath('descendant::*[re:match(name(), "meta", "i") and ' +
+    raster_cover_path = XPath('descendant::*[re:match(name(), "meta", "i") and '
             're:match(@name, "cover", "i") and @content]')
     guide_cover_path = XPath('descendant::*[local-name()="guide"]/*[local-name()="reference" and re:match(@type, "cover", "i")]/@href')
     identifier_path = XPath('descendant::*[re:match(name(), "identifier", "i")]')
-    application_id_path = XPath('descendant::*[re:match(name(), "identifier", "i") and '+
+    application_id_path = XPath('descendant::*[re:match(name(), "identifier", "i") and '
                             '(re:match(@opf:scheme, "calibre|libprs500", "i") or re:match(@scheme, "calibre|libprs500", "i"))]')
-    uuid_id_path    = XPath('descendant::*[re:match(name(), "identifier", "i") and '+
+    uuid_id_path    = XPath('descendant::*[re:match(name(), "identifier", "i") and '
                             '(re:match(@opf:scheme, "uuid", "i") or re:match(@scheme, "uuid", "i"))]')
     languages_path  = XPath('descendant::*[local-name()="language"]')
 
@@ -705,7 +705,7 @@ class OPF(object):  # {{{
     def itermanifest(self):
         return self.manifest_path(self.root)
 
-    def create_manifest_item(self, href, media_type):
+    def create_manifest_item(self, href, media_type, append=False):
         ids = [i.get('id', None) for i in self.itermanifest()]
         id = None
         for c in xrange(1, sys.maxint):
@@ -717,6 +717,9 @@ class OPF(object):  # {{{
         ans = etree.Element('{%s}item'%self.NAMESPACES['opf'],
                              attrib={'id':id, 'href':href, 'media-type':media_type})
         ans.tail = '\n\t\t'
+        if append:
+            manifest = self.manifest_ppath(self.root)[0]
+            manifest.append(ans)
         return ans
 
     def replace_manifest_item(self, item, items):
@@ -728,19 +731,6 @@ class OPF(object):  # {{{
         manifest[index:index+1] = items
         return [i.get('id') for i in items]
 
-    def add_path_to_manifest(self, path, media_type):
-        path = os.path.abspath(path)
-        for i in self.itermanifest():
-            xpath = os.path.join(self.base_dir, *(i.get('href', '').split('/')))
-            if os.path.abspath(xpath) == path:
-                return i.get('id')
-        href = os.path.relpath(path, self.base_dir).replace(os.sep, '/')
-        item = self.create_manifest_item(href, media_type)
-        manifest = self.manifest_ppath(self.root)[0]
-        manifest.append(item)
-        self.manifest.append_from_opf_manifest_item(item, self.basedir)
-        return item.get('id')
-
     def iterspine(self):
         return self.spine_path(self.root)
 
@@ -948,10 +938,24 @@ class OPF(object):  # {{{
                 return self.get_text(match) or None
 
         def fset(self, val):
+            uuid_id = None
+            for attr in self.root.attrib:
+                if attr.endswith('unique-identifier'):
+                    uuid_id = self.root.attrib[attr]
+                    break
+
             matches = self.isbn_path(self.metadata)
             if not val:
                 for x in matches:
-                    x.getparent().remove(x)
+                    xid = x.get('id', None)
+                    is_package_identifier = uuid_id is not None and uuid_id == xid
+                    if is_package_identifier:
+                        self.set_text(x, str(uuid.uuid4()))
+                        for attr in x.attrib:
+                            if attr.endswith('scheme'):
+                                x.attrib[attr] = 'uuid'
+                    else:
+                        x.getparent().remove(x)
                 return
             if not matches:
                 attrib = {'{%s}scheme'%self.NAMESPACES['opf']: 'ISBN'}
@@ -1149,6 +1153,11 @@ class OPF(object):  # {{{
                 if k == 'page-progression-direction' or k.endswith('}page-progression-direction'):
                     return v
 
+    @property
+    def primary_writing_mode(self):
+        for m in self.XPath('//*[local-name()="meta" and @name="primary-writing-mode" and @content]')(self.root):
+            return m.get('content')
+
     def guess_cover(self):
         '''
         Try to guess a cover. Needed for some old/badly formed OPF files.
@@ -1370,6 +1379,7 @@ class OPFCreator(Metadata):
         Metadata.__init__(self, title='', other=other)
         self.base_path = os.path.abspath(base_path)
         self.page_progression_direction = None
+        self.primary_writing_mode = None
         if self.application_id is None:
             self.application_id = str(uuid.uuid4())
         if not isinstance(self.toc, TOC):
@@ -1532,6 +1542,8 @@ class OPFCreator(Metadata):
             from calibre.ebooks.metadata.book.json_codec import object_to_unicode
             a(CAL_ELEM('calibre:user_categories',
                        json.dumps(object_to_unicode(self.user_categories))))
+        if self.primary_writing_mode:
+            a(M.meta(name='primary-writing-mode', content=self.primary_writing_mode))
         manifest = E.manifest()
         if self.manifest is not None:
             for ref in self.manifest:
